import moment from 'moment';

import { Selector, createSelector } from '@ngrx/store';
import { EntityState, createEntityAdapter, Dictionary } from '@ngrx/entity';

import { RoomUserDetailData } from '@ucap/protocol-sync';
import { RoomInfo, UserInfo as RoomUserInfo } from '@ucap/protocol-room';

export interface RoomState extends EntityState<RoomInfo> {
  syncDate: string;
}
export const adapterRoom = createEntityAdapter<RoomInfo>({
  selectId: roomInfo => roomInfo.roomId,
  sortComparer: (a, b) => {
    return (
      moment(b.finalEventDate)
        .toDate()
        .getTime() -
      moment(a.finalEventDate)
        .toDate()
        .getTime()
    );
  }
});

export interface RoomUserMap {
  roomId: string;
  userInfos: RoomUserInfo[];
}

export interface RoomUserState extends EntityState<RoomUserMap> {}
export const adapterRoomUser = createEntityAdapter<RoomUserMap>({
  selectId: roomUserDetailData => roomUserDetailData.roomId
});

export interface State {
  rooms: RoomState;
  roomUsers: RoomUserState;
}

const roomInitialState: RoomState = adapterRoom.getInitialState({
  syncDate: ''
});
const roomUserInitialState: RoomUserState = adapterRoomUser.getInitialState({});

export const initialState: State = {
  rooms: roomInitialState,
  roomUsers: roomUserInitialState
};

const {
  selectAll: selectAllForRoom,
  selectEntities: selectEntitiesForRoom,
  selectIds: selectIdsForRoom,
  selectTotal: selectTotalForRoom
} = adapterRoom.getSelectors();

const {
  selectAll: selectAllForRoomUser,
  selectEntities: selectEntitiesForRoomUser,
  selectIds: selectIdsForRoomUser,
  selectTotal: selectTotalForRoomUser
} = adapterRoomUser.getSelectors();

export function selectors<S>(selector: Selector<any, State>) {
  const selectRooms = createSelector(selector, (state: State) => state.rooms);

  const selectRoomUsers = createSelector(
    selector,
    (state: State) => state.roomUsers
  );

  return {
    rooms: createSelector(selectRooms, selectAllForRoom),
    room: createSelector(
      selectRooms,
      selectEntitiesForRoom,
      (roomState: RoomState, entities: Dictionary<RoomInfo>, roomId: string) =>
        entities && entities[roomId]
    ),
    roomsSyncDate: createSelector(selectRooms, roomState => roomState.syncDate),
    roomUsers: createSelector(selectRoomUsers, selectAllForRoomUser),
    roomUser: createSelector(
      selectRoomUsers,
      selectEntitiesForRoomUser,
      (
        roomUserState: RoomUserState,
        entities: Dictionary<RoomUserDetailData>,
        roomId: string
      ) => entities && entities[roomId]
    ),
    unreadTotal: createSelector(
      selectRooms,
      selectAllForRoom,
      (roomState: RoomState, rooms: RoomInfo[]) => {
        let unreadTotal = 0;
        for (const room of rooms) {
          unreadTotal += room.noReadCnt;
        }
        return unreadTotal;
      }
    )
  };
}
